@@ -1,4 +1,4 @@
-# $Id: Admin.pm,v 1.2 1998/12/18 01:18:18 eric Exp $
+# $Id: Admin.pm,v 1.14 1999/11/23 01:44:39 eric Exp $
 
 package IMAP::Admin;
 
@@ -6,6 +6,7 @@ use strict;
 use Carp;
 use IO::Select;
 use IO::Socket;
+use Text::ParseWords qw(quotewords);
 
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 
@@ -19,7 +20,7 @@ require AutoLoader;
 @EXPORT = qw(
 	
 );
-$VERSION = '0.5';
+$VERSION = '1.0.1';
 
 sub new {
     my $class = shift;
@@ -66,14 +67,25 @@ sub _initialize {
     $_ = <$fh>; # get Banner
     if (!/\* OK/) {
 	$self->close;
-	croak $_;
+	croak "$self->{'CLASS'}: Connection to $self->{'Server'} bad/no response: $!";
     }
-    print $fh "try login $self->{'Login'} $self->{'Password'}\n";
+    print $fh "try CAPABILITY\n";
+    $_ = <$fh>;
+    chomp;
+    if (/\r$/) {
+	chop;
+    }
+    $self->{'Capability'} = $_;
+    $_ = <$fh>;
+    if (!/^try OK/) {
+	croak "$self->{'CLASS'}: Couldn't do capabilites check";
+    }
+    print $fh "try LOGIN $self->{'Login'} $self->{'Password'}\n";
     $_ = <$fh>;
     if (/Login incorrect/) {
 	$self->close;
 	croak "$self->{'CLASS'}: Login incorrect while connecting to $self->{'Server'}";
-    } elsif (/try OK/) {
+    } elsif (/^try OK/) {
 	return;
     } else {
 	croak "$self->{'CLASS'}: Unknown error -- $_";
@@ -101,7 +113,7 @@ sub close {
 sub create {
     my $self = shift;
 
-    if (scalar(@_) != 1) {
+    if ((scalar(@_) != 1) && (scalar(@_) != 2)) {
 	$self->_error("create", "incorrect number of arguments");
 	return 1;
     }
@@ -111,7 +123,11 @@ sub create {
 	return 1;
     }
     my $fh = $self->{'Socket'};
-    print $fh "try CREATE $mailbox\n";
+    if (scalar(@_) == 1) { # a partition exists
+	print $fh "try CREATE $mailbox $_[0]\n";
+    } else {
+	print $fh "try CREATE $mailbox\n";
+    }
     $_ = <$fh>;
     if (/^try OK/) {
 	$self->{'Error'} = 'No Errors';
@@ -142,13 +158,57 @@ sub delete {
 	return 0;
     } else {
 	$self->_error("delete", "couldn't delete", $mailbox, ":", $_);
+	return 1;
+    }
+}
+sub get_quotaroot { # returns an array or undef
+    my $self = shift;
+    my (@quota, @info);
+
+    if (!($self->{'Capability'} =~ /QUOTA/)) {
+	$self->_error("get_quotaroot", "QUOTA not listed in server's capabilities");
+	return 1;
+    }
+    if (scalar(@_) != 1) {
+	$self->_error("get_quotaroot", "incorrect number of arguments");
+	return 1;
+    }
+    my $mailbox = shift;
+    if (!defined($self->{'Socket'})) {
+	$self->_error("get_quotaroot", "no connection open to", $self->{'Server'});
+	return 1;
+    }
+    my $fh = $self->{'Socket'};
+    print $fh "try GETQUOTAROOT $mailbox\n";
+    $_ = <$fh>;
+    while ((/\r$/) || (/\n$/)) {
+      chop;
+    }
+    while (/^\* QUOTA/) {
+	tr/\)\(//d;
+	@info = (split(' '))[2,4,5];
+	push @quota, @info;
+	$_ = <$fh>;
+        while ((/\r$/) || (/\n$/)) {
+          chop;
+        }
+    }
+    if (/^try OK/) {
+	return @quota;
+    } else {
+	$self->_error("get_quotaroot", "couldn't get quota for", $mailbox, ":", $_);
+	return;
     }
 }
 
-sub get_quota { # returns a hash or undef
+sub get_quota { # returns an array or undef
     my $self = shift;
-    my (%quota, @info);
+    my (@quota, @info);
 
+    if (!($self->{'Capability'} =~ /QUOTA/)) {
+	$self->_error("get_quota", "QUOTA not listed in server's capabilities");
+	return 1;
+    }
     if (scalar(@_) != 1) {
 	$self->_error("get_quota", "incorrect number of arguments");
 	return 1;
@@ -161,14 +221,20 @@ sub get_quota { # returns a hash or undef
     my $fh = $self->{'Socket'};
     print $fh "try GETQUOTA $mailbox\n";
     $_ = <$fh>;
+    while ((/\r$/) || (/\n$/)) {
+      chop;
+    }
     while (/^\* QUOTA/) {
 	tr/\)\(//d;
-	@info = (split(' '))[2,5];
-	$quota{$info[0]} = $info[1];
+	@info = (split(' '))[2,4,5];
+	push @quota, @info;
 	$_ = <$fh>;
+        while ((/\r$/) || (/\n$/)) {
+          chop;
+        }
     }
     if (/^try OK/) {
-	return %quota;
+	return @quota;
     } else {
 	$self->_error("get_quota", "couldn't get quota for", $mailbox, ":", $_);
 	return;
@@ -178,6 +244,10 @@ sub get_quota { # returns a hash or undef
 sub set_quota {
     my $self = shift;
 
+    if (!($self->{'Capability'} =~ /QUOTA/)) {
+	$self->_error("set_quota", "QUOTA not listed in server's capabilities");
+	return 1;
+    }
     if (scalar(@_) != 2) {
 	$self->_error("set_quota", "incorrect number of arguments");
 	return 1;
@@ -200,10 +270,14 @@ sub set_quota {
     }
 }
 
-sub get_acl { # returns a hash or undef
+sub get_acl { # returns an array or undef
     my $self = shift;
-    my (@info, %acl, $item);
+    my (@info, @acl_item, @acl, $item);
 
+    if (!($self->{'Capability'} =~ /ACL/)) {
+	$self->_error("get_acl", "ACL not listed in server's capabilities");
+	return 1;
+    }
     if (scalar(@_) != 1) {
 	$self->_error("get_acl", "incorrect number of arguments");
 	return;
@@ -216,13 +290,20 @@ sub get_acl { # returns a hash or undef
     my $fh = $self->{'Socket'};
     print $fh "try GETACL $mailbox\n";
     $_ = <$fh>;
+    while ((/\r$/) || (/\n$/)) {
+	chop;
+    }
     while (/^\* ACL/) {
 	@info = split(' ',$_,4);
-	$acl{$info[2]} = $info[3];
+        @acl_item = split(' ',$info[3]);
+	push @acl, @acl_item;
 	$_ = <$fh>;
+        while ((/\r$/) || (/\n$/)) {
+	    chop;
+        }
     }
     if (/^try OK/) {
-	return %acl;
+	return @acl;
     } else {
 	$self->_error("get_acl", "couldn't get acl for", $mailbox, ":", $_);
 	return;
@@ -233,6 +314,10 @@ sub set_acl {
     my $self = shift;
     my ($id, $acl);
 
+    if (!($self->{'Capability'} =~ /ACL/)) {
+	$self->_error("set_acl", "ACL not listed in server's capabilities");
+	return 1;
+    }
     if (scalar(@_) < 2) {
 	$self->_error("set_acl", "too few arguments");
 	return 1;
@@ -266,6 +351,10 @@ sub delete_acl {
     my $self = shift;
     my ($id, $acl);
 
+    if (!($self->{'Capability'} =~ /ACL/)) {
+	$self->_error("delete_acl", "ACL not listed in server's capabilities");
+	return 1;
+    }
     if (scalar(@_) < 1) {
 	$self->_error("delete_acl", "incorrect number of arguments");
 	return 1;
@@ -303,12 +392,18 @@ sub list { # wild cards are allowed, returns array or undef
 	return;
     }
     my $fh = $self->{'Socket'};
-    print $fh "try LIST $list $list\n";
+    print $fh "try LIST \"\" \"$list\"\n";
     $_ = <$fh>;
+    while ((/\r$/) || (/\n$/)) {
+      chop;
+    }
     while (/\* /) { # danger danger (could lock up needs timeout)
-	@info = split(' ');
+	@info = quotewords('\s+', 0, $_);
 	push @mail, $info[$#info];
 	$_ = <$fh>;
+        while ((/\r$/) || (/\n$/)) {
+          chop;
+        }
     }
     if (/^try OK/) {
 	return @mail;
@@ -341,25 +436,29 @@ IMAP::Admin - Perl module for basic IMAP server administration
   if ($err != 0) {
     print "$imap->{'Error'}\n";
   }
+  $err = $imap->create("user.bob", "green"); 
   $err = $imap->delete("user.bob");
 
-  %quota = $imap->get_quota("user.bob");
+  @quota = $imap->get_quotaroot("user.bob");
+  @quota = $imap->get_quota("user.bob");
   $err = $imap->set_quota("user.bob", 10000);
 
-  %acl = $imap->get_acl("user.bob");
+  @acl = $imap->get_acl("user.bob");
   $err = $imap->set_acl("user.bob", "admin", "lrswipdca", "joe", "lrs");
   $err = $imap->delete_acl("user.bob", "joe", "admin");
  
   @list = $imap->list("user.bob");
   @list = $imap->list("user.b*");
 
-  
+  $imap->{'Capability'} # this contains the Capabilities reply from the IMAP server
+
+  $imap->close; # close open imap connection
 
 =head1 DESCRIPTION
 
 IMAP::Admin provides basic IMAP server adminstration.  It provides functions for creating and deleting mailboxes and setting various information such as quotas and access rights.
 
-It's interface should, in theory, work with any RFC compliant IMAP server, but I currently have only tested it against Carnegie Mellon University's Cyrus IMAP.
+It's interface should, in theory, work with any RFC compliant IMAP server, but I currently have only tested it against Carnegie Mellon University's Cyrus IMAP.  It does a CAPABILITY check for Cyrus specific extensions to see if they are supported.
 
 Operationally it opens a socket connection to the IMAP server and logs in with the supplied login and password.  You then can call any of the functions to perform their associated operation.
 
@@ -369,24 +468,22 @@ Operationally it opens a socket connection to the IMAP server and logs in with t
 RFC2060 commands.  These should work with any RFC2060 compliant IMAP mail servers.
 
 create makes new mailboxes.  Cyrus IMAP, for normal mailboxes, has the user. prefix.
-create returns a 0 on success or a 1 on failure.  An error message is placed in the
-object->{'Error'} variable on failure.
+create returns a 0 on success or a 1 on failure.  An error message is placed in the object->{'Error'} variable on failure. create takes an optional second argument that is the partition to create the mailbox in (this might be a Cyrus specific extension).
 
 delete destroys mailboxes.
-delete returns a 0 on success or a 1 on failure.  An error message is placed in the
-object->{'Error'} variable on failure.
+delete returns a 0 on success or a 1 on failure.  An error message is placed in the object->{'Error'} variable on failure.
 
 list lists mailboxes.  list accepts wildcard matching
 
-Currently create and delete only take one argument.  This will probably change in the future to allow for mass creation/destruction.
 
 =head2 QUOTA FUNCTIONS
 
 NOT RFC2060 commands.  I believe these are specific to Cyrus IMAP.
 
-get_quota retrieves quota information.  It returns a hash on success and undef on failure.  In the event of a failure the error is place in the object->{'Error'} variable.
-
-get_quota takes only one argument, but this will probably change to multiple and/or wildcard matching.
+get_quotaroot and get_quota retrieve quota information.  They return an array on success and undef on failure.  In the event of a failure the error is place in the object->{'Error'} variable.  The array has three elements for each item in the quota. 
+$quota[0] <- mailbox name
+$quota[1] <- quota amount used in kbytes
+$quota[2] <- quota in kbytes
 
 set_quota sets the quota.  The number is in kilobytes so 10000 is approximately 10Meg.
 set_quota returns a 0 on success or a 1 on failure.  An error message is placed in the object->{'Error'} variable on failure.
@@ -395,7 +492,10 @@ set_quota returns a 0 on success or a 1 on failure.  An error message is placed
 
 NOT RFC2060 commands.  I believe these are specific to Cyrus IMAP.
 
-get_acl retrieves acl information.  It returns a hash on success and under on failure.  In the event of a failure the error is placed in the object->{'Error'} variable.
+get_acl retrieves acl information.  It returns an array on success and under on failure.  In the event of a failure the error is placed in the object->{'Error'} variable. The array contains a pair for each person who has an acl on this mailbox
+$acl[0] user who has acl information
+$acl[1] acl information
+$acl[2] next user ...
 
 set_acl set acl information for a single mailbox.  You can specify more the one user's rights on the same set call.  It returns a 0 on success or a 1 on failure.  An error message is placed in the object->{'Error'} variable on failure.
 
@@ -414,11 +514,11 @@ Currently all the of the socket traffic is handled via prints and <>.  This mean
 
 =head1 CVS REVISION
 
-$Id: Admin.pm,v 1.2 1998/12/18 01:18:18 eric Exp $
+$Id: Admin.pm,v 1.14 1999/11/23 01:44:39 eric Exp $
 
 =head1 AUTHOR
 
-Eric Estabrooks, estabroo@ispn.com
+Eric Estabrooks, eric@urbanrage.com
 
 =head1 SEE ALSO
 
